#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# (c) Rene Hadler
# rene.hadler@iteas.at
# www.iteas.at
#
import json

import msal
import requests
from MAPI.Util import *
import xmlrpc.client
import configparser
from datetime import timedelta, datetime
import ssl

config = configparser.ConfigParser()
config.read('/etc/odoo2o365-tools/calendar.conf')

O365_AUTHORITY = config.get("IMPORT", "O365_AUTHORITY")
O365_CLIENT_ID = config.get("IMPORT", "O365_CLIENT_ID")
O365_SCOPE = config.get("IMPORT", "O365_SCOPE")
O365_SECRET = config.get("IMPORT", "O365_SECRET")
O365_CAL_USER = config.get("IMPORT", "O365_CAL_USER")
O365_CAL = config.get("IMPORT", "O365_CAL")

ODOO_SERVER = config.get("EXPORT", "ODOO_SERVER")
ODOO_DB = config.get("EXPORT", "ODOO_DB")
ODOO_API_USER = config.get("EXPORT", "ODOO_API_USER")
ODOO_API_PASS = config.get("EXPORT", "ODOO_API_PASS")
ODOO_API_LANG = config.get("EXPORT", "ODOO_API_LANG")

SYNC_START = config.get("SYNC", "START")
SYNC_END = config.get("SYNC", "END")

o365app = msal.ConfidentialClientApplication(O365_CLIENT_ID, authority=O365_AUTHORITY, client_credential=O365_SECRET)
o365result = o365app.acquire_token_for_client(scopes=[O365_SCOPE])

if "access_token" not in o365result:
    print("Got no O365 Access token, check your configuration, exiting")
    sys.exit(1)

api_common = xmlrpc.client.ServerProxy('%s/xmlrpc/2/common' % ODOO_SERVER, context=ssl.create_default_context(ssl.Purpose.SERVER_AUTH))
uid = api_common.authenticate(ODOO_DB, ODOO_API_USER, ODOO_API_PASS, {})

api_models = xmlrpc.client.ServerProxy('%s/xmlrpc/2/object' % ODOO_SERVER, context=ssl.create_default_context(ssl.Purpose.SERVER_AUTH))
#
try:
    data = api_models.execute_kw(
        ODOO_DB, uid, ODOO_API_PASS, 'calendar.event', 'search_read',
        [[['start', '>=', SYNC_START], ['stop', '<=', SYNC_END]]],
        {
         'fields': ['name', 'description', 'location', 'start', 'user_id', 'duration', 'alarm_ids', 'categ_ids', 'partner_ids', 'stop', 'allday'],
         'context' :{'lang': ODOO_API_LANG}
        })
except xmlrpc.client.Fault:
    print("Fehler in der Odoo Datenabfrage!", sys.exc_info())
    sys.exit(1)

#
try:
    data2 = api_models.execute_kw(
        ODOO_DB, uid, ODOO_API_PASS, 'res.users', 'search_read',
        [],
        {
            'fields': ['login', 'partner_id'],
            'context' :{'lang': ODOO_API_LANG}
        })

    #user_ids = []
    #for user in data2:
    #    if user["partner_id"][0] not in user_ids:
    #        user_ids.append(user["partner_id"][0])

    data2_partner = api_models.execute_kw(
        ODOO_DB, uid, ODOO_API_PASS, 'res.partner', 'search_read',
        [],
        {
            'fields': ['email', 'name', 'parent_id'],
            'context' :{'lang': ODOO_API_LANG}
        })

    for user in data2:
        for partner in data2_partner:
            if user["partner_id"][0] == partner["id"]:
                user["partner"] = partner

except xmlrpc.client.Fault:
    print("Fehler in der Odoo Datenabfrage!", sys.exc_info())
    sys.exit(1)

try:
    data3 = api_models.execute_kw(
        ODOO_DB, uid, ODOO_API_PASS, 'calendar.alarm', 'search_read',
        [],
        {
            'fields': ['duration_minutes'],
            'context' :{'lang': ODOO_API_LANG}
        })
except xmlrpc.client.Fault:
    print("Fehler in der Odoo Datenabfrage!", sys.exc_info())
    sys.exit(1)

try:
    data4 = api_models.execute_kw(
        ODOO_DB, uid, ODOO_API_PASS, 'calendar.event.type', 'search_read',
        [],
        {
            'fields': ['id', 'display_name'],
            'context' :{'lang': ODOO_API_LANG}
        })
except xmlrpc.client.Fault:
    print("Fehler in der Odoo Datenabfrage!", sys.exc_info())
    sys.exit(1)


categories = {}
for category in data4:
    categories[category['id']] = category['display_name']

#tt = requests.get("https://graph.microsoft.com/v1.0/users/%s/calendars/" % (O365_CAL_USER), headers={'Authorization': 'Bearer ' + o365result['access_token'], 'Prefer': 'outlook.timezone="Europe/Berlin"'}, params={'$top': "10000"}).json()
#print(json.dumps(tt, indent=2))
#sys.exit(1)

o365_events = requests.get("https://graph.microsoft.com/v1.0/users/%s/calendars/%s/events" % (O365_CAL_USER, O365_CAL), headers={'Authorization': 'Bearer ' + o365result['access_token'], 'Prefer': 'outlook.timezone="Europe/Berlin"'}, params={'$top': "10000", '$select': 'exteh7ibh8m_iteasodoo'}).json()
#print(json.dumps(o365_events["value"][0], indent=2))

o365_odoo_schema = {
    "id": "iteasodoo",
    "description": "iteas Odoo schema extension",
    "targetTypes": [
        "Event"
    ],
    "properties": [
        {
            "name": "eventId",
            "type": "String"
        },
    ]
}

#requests.delete("https://graph.microsoft.com/v1.0/schemaExtensions/ext2yrob1y4_iteasodoo", headers={'Authorization': 'Bearer ' + o365result['access_token']})
#requests.delete("https://graph.microsoft.com/v1.0/schemaExtensions/extpfk4vygo_iteasodoo", headers={'Authorization': 'Bearer ' + o365result['access_token']}).json()

#o365_schema = requests.post("https://graph.microsoft.com/v1.0/schemaExtensions", headers={'Authorization': 'Bearer ' + o365result['access_token']}, json=o365_odoo_schema).json()
#print("o365 schema", o365_schema)

o365ext = requests.get("https://graph.microsoft.com/v1.0/schemaExtensions", headers={'Authorization': 'Bearer ' + o365result['access_token']}, params={'$top': "1000", "$filter": "description eq 'iteas Odoo schema extension'"}).json()
if len(o365ext["value"]) == 0:
    print("O365 Odoo-Schema Extension nicht gefunden!")
    sys.exit(1)

O365_SCHEMA = o365ext["value"][0]["id"]

map_oid2entryid = {}
# Remove deleted events routines
delids = []
if len(o365_events["value"]) > 0:
    for item in o365_events["value"]:
        if O365_SCHEMA in item:
            iid = int(item[O365_SCHEMA]["eventId"])
            if iid:
                delids.append(iid)
                map_oid2entryid[iid] = item["id"]

for event in data:
    if event['name'] == "Busy":
        try:
            event['name'] = "Busy (%s)" % event["user_id"][1]
        except:
            pass

    if event['id'] in delids:
        delids.remove(event['id'])

print("Starting odoo2o365-calendar cleanup")
for item in o365_events["value"]:
    if O365_SCHEMA in item:
        iid = int(item[O365_SCHEMA]["eventId"])
        if iid:
            if iid in delids:
                print("Delete calendar event", item)
                requests.delete("https://graph.microsoft.com/v1.0/users/%s/calendars/%s/events/%s" % (O365_CAL_USER, O365_CAL, item["id"]), headers={'Authorization': 'Bearer ' + o365result['access_token']})


print("Starting odoo2o365-calendar import with total events: ", len(data))
batch_requests = []
for event in data:

    if event['id'] in map_oid2entryid:
        #new_item = list(filter(lambda oevent: oevent['id'] == map_oid2entryid[event['id']], o365_events["value"]))[0]
        new_item = {}
        new_item["id"] = map_oid2entryid[event['id']]
    else:
        new_item = {}

    #organizer = list(filter(lambda x: x["id"] == event["user_id"][0], data2))[0]
    #new_item["isOrganizer"] = False
    #new_item["organizer"] = {
    #    "emailAddress": {
    #      "name": organizer["partner"]["name"],
    #      "address": organizer["partner"]["email"]
    #    }
    #  }
    new_item["responseRequested"] = False

    if event['categ_ids']:
        ic = []
        for catid in event['categ_ids']:
            ic.append(categories[catid])

        new_item["categories"] = ic

    n_attendees = []
    if event['partner_ids']:
        for attendee_id in event['partner_ids']:
            attendee = list(filter(lambda x: x['id'] == attendee_id, data2_partner))[0]
            addr_name = attendee["email"] if attendee["email"] else attendee["name"]
            n_attendees.append({
                "emailAddress": {
                    "address": attendee["email"] if attendee["email"] else "",
                    "name": addr_name
                },
                "type": "required"
            })

    new_item["attendees"] = n_attendees
    new_item["subject"] = event['name']
    new_item["body"] = {
        "contentType": "html",
        "content": event['description'] if event['description'] else ""
    }
    if(event["location"]):
        new_item["location"] = {
            "displayName": event['location'],
            "locationType": "default",
            "uniqueId": event['location'],
            "uniqueIdType": "private"
          }

    if event['allday']:
        new_item["isAllDay"] = True

        adate_start = datetime.strptime(event['start'], "%Y-%m-%d %H:%M:%S")
        adate_end = datetime.strptime(event['stop'], "%Y-%m-%d %H:%M:%S") + timedelta(days=1)
        adate_start_f = datetime(adate_start.year, adate_start.month, adate_start.day, 0, 0, 0)
        adate_end_f = datetime(adate_end.year, adate_end.month, adate_end.day, 0, 0, 0)
        new_item["start"] = {
            "dateTime": adate_start_f.isoformat(),
            "timeZone": "Europe/Berlin"
        }
        new_item["end"] = {
            "dateTime": adate_end_f.isoformat(),
            "timeZone": "Europe/Berlin"
        }
    else:
        new_item["isAllDay"] = False

        time_start_date = datetime.strptime(event['start'], "%Y-%m-%d %H:%M:%S")
        new_item["start"] = {
            "dateTime": time_start_date.isoformat(),
            "timeZone": "UTC"
        }

        time_end_date = datetime.strptime(event['stop'], "%Y-%m-%d %H:%M:%S")
        new_item["end"] = {
            "dateTime": time_end_date.isoformat(),
            "timeZone": "UTC"
        }

    if event['alarm_ids']:
        alarmid = event['alarm_ids'][0]
        alarms = list(filter(lambda x: x['id'] == alarmid, data3))
        if alarms:
            alarm = alarms[0]
            new_item["isReminderOn"] = True
            new_item["reminderMinutesBeforeStart"] = alarm['duration_minutes']
    else:
        new_item["isReminderOn"] = False
        new_item["reminderMinutesBeforeStart"] = 0

    if event['id'] in map_oid2entryid:
        new_item.pop(O365_SCHEMA, None)
        #tmp2 = requests.patch(
        #    "https://graph.microsoft.com/v1.0/users/%s/calendars/%s/events/%s" % (O365_CAL_USER, O365_CAL, new_item["id"]),
        #    headers={'Authorization': 'Bearer ' + o365result['access_token']}, json=new_item).json()
        batch_requests.append({
            "id": str(len(batch_requests)+1),
            "method": "PATCH",
            "url": "/users/%s/calendars/%s/events/%s" % (O365_CAL_USER, O365_CAL, new_item["id"]),
            "headers": {
                "Content-Type": "application/json"
            },
            "body": new_item
        })

        if len(batch_requests) > 18:
            #print(json.dumps(batch_requests, indent=2))
            tmp2 = requests.post(
                "https://graph.microsoft.com/v1.0/$batch",
                headers={'Authorization': 'Bearer ' + o365result['access_token']}, json={"requests": batch_requests}).json()
            batch_requests.clear()

        print("Update Odoo event successful: %s" % (event['name']))
    else:
        tmp = requests.post("https://graph.microsoft.com/v1.0/users/%s/calendars/%s/events" % (O365_CAL_USER, O365_CAL),
                            headers={'Authorization': 'Bearer ' + o365result['access_token']}, json=new_item).json()

        patch_item = {
            O365_SCHEMA: {
                "eventId": str(event['id'])
            }
        }
        tmp2 = requests.patch("https://graph.microsoft.com/v1.0/users/%s/calendars/%s/events/%s" % (O365_CAL_USER, O365_CAL, tmp["id"]),
                            headers={'Authorization': 'Bearer ' + o365result['access_token']}, json=patch_item).json()

        print("Import Odoo event successful: %s" % (event['name']))

if len(batch_requests) > 0:
    tmp2 = requests.post(
        "https://graph.microsoft.com/v1.0/$batch",
        headers={'Authorization': 'Bearer ' + o365result['access_token']}, json={"requests": batch_requests}).json()